@@ -16,8 +16,8 @@
 from unittest import mock
 
 import contextlib
+import os
 from os import stat as orig_stat
-import six
 import stat
 import sys
 
@@ -36,6 +36,7 @@ class FakeSelinux(object):
 
 sys.modules["selinux"] = FakeSelinux
 
+from container_config_scripts.nova_statedir_ownership import get_exclude_paths  # noqa: E402
 from container_config_scripts.nova_statedir_ownership import \
     NovaStatedirOwnershipManager  # noqa: E402
 from container_config_scripts.nova_statedir_ownership import PathManager  # noqa: E402
@@ -373,7 +374,7 @@ class NovaStatedirOwnershipManagerTestCase(base.BaseTestCase):
 
         # Determine which paths should change uid/gid
         expected_changes = {}
-        for k, v in six.iteritems(testtree):
+        for k, v in testtree.items():
             if k == '/var/lib/nova/upgrade_marker':
                 # Ignore the marker, it should be deleted
                 continue
@@ -392,6 +393,34 @@ class NovaStatedirOwnershipManagerTestCase(base.BaseTestCase):
 
         with fake_testtree(testtree) as (_, _, _, _, fake_unlink, _, _):
             NovaStatedirOwnershipManager('/var/lib/nova').run()
-            for fn, expected in six.iteritems(expected_changes):
+            for fn, expected in expected_changes.items():
                 assert_ids(testtree, fn, expected[0], expected[1])
             fake_unlink.assert_called_with('/var/lib/nova/upgrade_marker')
+
+    def test_exclude_path(self):
+        testtree = generate_testtree1(current_uid, current_gid)
+
+        with fake_testtree(testtree) as (
+                fake_chown, _, fake_listdir, fake_stat, _, _, _):
+            manager = NovaStatedirOwnershipManager(
+                '/var/lib/nova',
+                exclude_paths=['instances/foo/bar', '/var/lib/nova/instances/foo/removeddir']
+            )
+            manager.run()
+            self.assertIn('/var/lib/nova/instances/foo/bar', manager.exclude_paths)
+            self.assertIn('/var/lib/nova/instances/foo/removeddir', manager.exclude_paths)
+            self.assertNotIn(mock.call('/var/lib/nova/instances/foo/bar'), fake_stat.call_args_list)
+            self.assertNotIn(mock.call('/var/lib/nova/instances/foo/bar'), fake_chown.call_args_list)
+            self.assertNotIn(mock.call('/var/lib/nova/instances/foo/removeddir'), fake_stat.call_args_list)
+            self.assertNotIn(mock.call('/var/lib/nova/instances/foo/removeddir'), fake_chown.call_args_list)
+            self.assertNotIn(mock.call('/var/lib/nova/instances/foo/removeddir'), fake_listdir.call_args_list)
+
+    @mock.patch.dict(os.environ, {'NOVA_STATEDIR_OWNERSHIP_SKIP': 'foo:bar:foo/bar/baz'})
+    def test_get_exclude_paths(self):
+        expected = [
+            'foo',
+            'bar',
+            'foo/bar/baz'
+        ]
+        exclude_paths = get_exclude_paths()
+        self.assertEqual(exclude_paths, expected)
